#ifndef GRAYHISTOGRAM_H
#define GRAYHISTOGRAM_H

#include <vector>
#include <numeric>

namespace grayASMS {

#define VIZINHOS 1

class Bin;

class Histogram
{
public:
    std::vector<double> data;
    int dimSize;
    int range;
    int rangePerBin;
    float rangePerBinInv;
    bool initialized;
    unsigned int lookUpTable1[256];
    float pesos[VIZINHOS*2 +1];
    int area;


    Histogram(int _dimSize = 128, int _range = 256);
    ~Histogram(){}

    void insertValues(std::vector<unsigned char> & data1,
                      std::vector<double> & weight);

    double computeSimilarity(Histogram * hist);

    double getValue(int val1);
    void addValue(int val1, double weight);

    double getMin();
    void transformToWeights();
    void transformByWeight(double min);
    void multiplyByWeights(Histogram * hist);
    void adapt(Histogram * hist, float height);

    void clear();
    void normalize();
    void addExpHist(double alpha, Histogram & hist);

};

}

#endif //HISTOGRAM_H
